@@ -38,8 +38,8 @@ Exported symbols include:
    As part of the type-hierarchy:    xx -- is bit-width
 
    generic
-     +-> bool_
-     +-> number
+     +-> bool_                                  (kind=b)
+     +-> number                                 (kind=i)
      |     integer
      |     signedinteger   (intxx)
      |     byte
@@ -48,7 +48,7 @@ Exported symbols include:
      |     intp           int0
      |     int_
      |     longlong
-     +-> unsignedinteger  (uintxx)
+     +-> unsignedinteger  (uintxx)              (kind=u)
      |     ubyte
      |     ushort
      |     uintc
@@ -56,29 +56,27 @@ Exported symbols include:
      |     uint_
      |     ulonglong
      +-> inexact
-     |   +-> floating           (floatxx)
+     |   +-> floating           (floatxx)       (kind=f)
      |   |     single
      |   |     float_  (double)
      |   |     longfloat
-     |   \-> complexfloating    (complexxx)
+     |   \-> complexfloating    (complexxx)     (kind=c)
      |         csingle  (singlecomplex)
      |         complex_ (cfloat, cdouble)
      |         clongfloat (longcomplex)
      +-> flexible
      |     character
-     |     str_     (string_)
-     |     unicode_
-     |     void
+     |     str_     (string_)                   (kind=S)
+     |     unicode_                             (kind=U)
+     |     void                                 (kind=V)
      |
-     \-> object_ (not used much)
-
-$Id: numerictypes.py,v 1.17 2005/09/09 22:20:06 teoliphant Exp $
+     \-> object_ (not used much)                (kind=O)
 """
 
 # we add more at the bottom
 __all__ = ['sctypeDict', 'sctypeNA', 'typeDict', 'typeNA', 'sctypes',
            'ScalarType', 'obj2sctype', 'cast', 'nbytes', 'sctype2char',
-           'maximum_sctype', 'issctype', 'typecodes']
+           'maximum_sctype', 'issctype', 'typecodes', 'find_common_type']
 
 from numpy.core.multiarray import typeinfo, ndarray, array, empty, dtype
 import types as _types
@@ -87,6 +85,111 @@ import types as _types
 # as numerictypes.bool, etc.
 from __builtin__ import bool, int, long, float, complex, object, unicode, str
 
+
+# String-handling utilities to avoid locale-dependence.
+
+# "import string" is costly to import!
+# Construct the translation tables directly
+#   "A" = chr(65), "a" = chr(97)
+_all_chars = map(chr, range(256))
+_ascii_upper = _all_chars[65:65+26]
+_ascii_lower = _all_chars[97:97+26]
+LOWER_TABLE="".join(_all_chars[:65] + _ascii_lower + _all_chars[65+26:])
+UPPER_TABLE="".join(_all_chars[:97] + _ascii_upper + _all_chars[97+26:])
+
+#import string
+# assert (string.maketrans(string.ascii_uppercase, string.ascii_lowercase) == \
+#          LOWER_TABLE)
+# assert (string.maketrnas(string_ascii_lowercase, string.ascii_uppercase) == \
+#          UPPER_TABLE)
+#LOWER_TABLE = string.maketrans(string.ascii_uppercase, string.ascii_lowercase)
+#UPPER_TABLE = string.maketrans(string.ascii_lowercase, string.ascii_uppercase)
+
+def english_lower(s):
+    """ Apply English case rules to convert ASCII strings to all lower case.
+
+    This is an internal utility function to replace calls to str.lower() such
+    that we can avoid changing behavior with changing locales. In particular,
+    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
+    both lowercase and uppercase. Thus, "I".lower() != "i" in a "tr" locale.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    lowered : str
+
+    Examples
+    --------
+    >>> from numpy.core.numerictypes import english_lower
+    >>> english_lower('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
+    'abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz0123456789_'
+    >>> english_lower('')
+    ''
+    """
+    lowered = s.translate(LOWER_TABLE)
+    return lowered
+
+def english_upper(s):
+    """ Apply English case rules to convert ASCII strings to all upper case.
+
+    This is an internal utility function to replace calls to str.upper() such
+    that we can avoid changing behavior with changing locales. In particular,
+    Turkish has distinct dotted and dotless variants of the Latin letter "I" in
+    both lowercase and uppercase. Thus, "i".upper() != "I" in a "tr" locale.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    uppered : str
+
+    Examples
+    --------
+    >>> from numpy.core.numerictypes import english_upper
+    >>> english_upper('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_')
+    'ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_'
+    >>> english_upper('')
+    ''
+    """
+    uppered = s.translate(UPPER_TABLE)
+    return uppered
+
+def english_capitalize(s):
+    """ Apply English case rules to convert the first character of an ASCII
+    string to upper case.
+
+    This is an internal utility function to replace calls to str.capitalize()
+    such that we can avoid changing behavior with changing locales.
+
+    Parameters
+    ----------
+    s : str
+
+    Returns
+    -------
+    capitalized : str
+
+    Examples
+    --------
+    >>> from numpy.core.numerictypes import english_capitalize
+    >>> english_capitalize('int8')
+    'Int8'
+    >>> english_capitalize('Int8')
+    'Int8'
+    >>> english_capitalize('')
+    ''
+    """
+    if s:
+        return english_upper(s[0]) + s[1:]
+    else:
+        return s
+
+
 sctypeDict = {}      # Contains all leaf-node scalar types with aliases
 sctypeNA = {}        # Contails all leaf-node types -> numarray type equivalences
 allTypes = {}      # Collect the types we will add to the module here
@@ -114,7 +217,7 @@ def bitname(obj):
             newname = name[:-1]
         else:
             newname = name
-        info = typeinfo[newname.upper()]
+        info = typeinfo[english_upper(newname)]
         assert(info[-1] == obj)  # sanity check
         bits = info[2]
 
@@ -149,7 +252,7 @@ def bitname(obj):
 
 def _add_types():
     for a in typeinfo.keys():
-        name = a.lower()
+        name = english_lower(a)
         if isinstance(typeinfo[a], tuple):
             typeobj = typeinfo[a][-1]
 
@@ -165,7 +268,7 @@ _add_types()
 
 def _add_aliases():
     for a in typeinfo.keys():
-        name = a.lower()
+        name = english_lower(a)
         if not isinstance(typeinfo[a], tuple):
             continue
         typeobj = typeinfo[a][-1]
@@ -179,12 +282,12 @@ def _add_aliases():
                 allTypes[myname] = typeobj
                 sctypeDict[myname] = typeobj
                 if base == 'complex':
-                    na_name = '%s%d' % (base.capitalize(), bit/2)
+                    na_name = '%s%d' % (english_capitalize(base), bit/2)
                 elif base == 'bool':
-                    na_name = base.capitalize()
+                    na_name = english_capitalize(base)
                     sctypeDict[na_name] = typeobj
                 else:
-                    na_name = "%s%d" % (base.capitalize(), bit)
+                    na_name = "%s%d" % (english_capitalize(base), bit)
                     sctypeDict[na_name] = typeobj
                 sctypeNA[na_name] = typeobj
                 sctypeDict[na_name] = typeobj
@@ -394,6 +497,49 @@ def obj2sctype(rep, default=None):
     return res.type
 
 
+def issubclass_(arg1, arg2):
+    try:
+        return issubclass(arg1, arg2)
+    except TypeError:
+        return False
+
+def issubsctype(arg1, arg2):
+    return issubclass(obj2sctype(arg1), obj2sctype(arg2))
+
+def issubdtype(arg1, arg2):
+    """
+    Returns True if first argument is a typecode lower/equal in type hierarchy.
+
+    Parameters
+    ----------
+    arg1 : dtype_like
+        dtype or string representing a typecode.
+    arg2 : dtype_like
+        dtype or string representing a typecode.
+
+
+    See Also
+    --------
+    numpy.core.numerictypes : Overview of numpy type hierarchy.
+
+    Examples
+    --------
+    >>> np.issubdtype('S1', str)
+    True
+    >>> np.issubdtype(np.float64, np.float32)
+    False
+
+    """
+    if issubclass_(arg2, generic):
+        return issubclass(dtype(arg1).type, arg2)
+    mro = dtype(arg2).type.mro()
+    if len(mro) > 1:
+        val = mro[1]
+    else:
+        val = mro[0]
+    return issubclass(dtype(arg1).type, val)
+
+
 # This dictionary allows look up based on any alias for an array data-type
 class _typedict(dict):
     def __getitem__(self, obj):
@@ -438,9 +584,6 @@ ScalarType = tuple(ScalarType)
 for key in _sctype2char_dict.keys():
     cast[key] = lambda x, k=key : array(x, copy=False).astype(k)
 
-
-_unicodesize = array('u','U1').itemsize
-
 # Create the typestring lookup dictionary
 _typestr = _typedict()
 for key in _sctype2char_dict.keys():
@@ -474,7 +617,7 @@ for key in allTypes:
 
 del key
 
-typecodes = {'Character':'S1',
+typecodes = {'Character':'c',
              'Integer':'bhilqp',
              'UnsignedInteger':'BHILQP',
              'Float':'fdg',
@@ -486,3 +629,84 @@ typecodes = {'Character':'S1',
 # backwards compatibility --- deprecated name
 typeDict = sctypeDict
 typeNA = sctypeNA
+
+# b -> boolean
+# u -> unsigned integer
+# i -> signed integer
+# f -> floating point
+# c -> complex
+# S -> string
+# U -> Unicode string
+# V -> record
+# O -> Python object
+_kind_list = ['b', 'u', 'i', 'f', 'c', 'S', 'U', 'V', 'O']
+
+__test_types = typecodes['AllInteger'][:-2]+typecodes['AllFloat']+'O'
+__len_test_types = len(__test_types)
+
+# Keep incrementing until a common type both can be coerced to
+#  is found.  Otherwise, return None
+def _find_common_coerce(a, b):
+    if a > b:
+        return a
+    try:
+        thisind = __test_types.index(a.char)
+    except ValueError:
+        return None
+    while thisind < __len_test_types:
+        newdtype = dtype(__test_types[thisind])
+        if newdtype >= b and newdtype >= a:
+            return newdtype
+        thisind += 1
+    return None
+
+
+def find_common_type(array_types, scalar_types):
+    """
+    Determine common type following standard coercion rules
+
+    Parameters
+    ----------
+    array_types : sequence
+        A list of dtype convertible objects representing arrays
+    scalar_types : sequence
+        A list of dtype convertible objects representing scalars
+
+    Returns
+    -------
+    datatype : dtype
+        The common data-type which is the maximum of the array_types
+        ignoring the scalar_types unless the maximum of the scalar_types
+        is of a different kind.
+
+        If the kinds is not understood, then None is returned.
+
+    See Also
+    --------
+    dtype
+
+    """
+    array_types = [dtype(x) for x in array_types]
+    scalar_types = [dtype(x) for x in scalar_types]
+
+    if len(scalar_types) == 0:
+        if len(array_types) == 0:
+            return None
+        else:
+            return max(array_types)
+    if len(array_types) == 0:
+        return max(scalar_types)
+
+    maxa = max(array_types)
+    maxsc = max(scalar_types)
+
+    try:
+        index_a = _kind_list.index(maxa.kind)
+        index_sc = _kind_list.index(maxsc.kind)
+    except ValueError:
+        return None
+
+    if index_sc > index_a:
+        return _find_common_coerce(maxsc,maxa)
+    else:
+        return maxa
