@@ -1,3 +1,39 @@
+"""
+Record Arrays
+=============
+Record arrays expose the fields of structured arrays as properties.
+
+Most commonly, ndarrays contain elements of a single type, e.g. floats, integers,
+bools etc.  However, it is possible for elements to be combinations of these,
+such as::
+
+  >>> a = np.array([(1, 2.0), (1, 2.0)], dtype=[('x', int), ('y', float)])
+  >>> a
+  array([(1, 2.0), (1, 2.0)],
+        dtype=[('x', '<i4'), ('y', '<f8')])
+
+Here, each element consists of two fields: x (and int), and y (a float).
+This is known as a structured array.  The different fields are analogous
+to columns in a spread-sheet.  The different fields can be accessed as
+one would a dictionary::
+
+  >>> a['x']
+  array([1, 1])
+
+  >>> a['y']
+  array([ 2.,  2.])
+
+Record arrays allow us to access fields as properties::
+
+  >>> ar = a.view(np.recarray)
+
+  >>> ar.x
+  array([1, 1])
+
+  >>> ar.y
+  array([ 2.,  2.])
+
+"""
 # All of the functions allow formats to be a dtype
 __all__ = ['record', 'recarray', 'format_parser']
 
@@ -43,10 +79,40 @@ def find_duplicate(list):
     return dup
 
 class format_parser:
+    """Class to convert formats, names, titles description to a dtype
+
+    After constructing the format_parser object, the dtype attribute is
+      the converted data-type.
+
+    dtype = format_parser(formats, names, titles).dtype
+
+    Parameters
+    ----------
+    formats : string or list
+        comma-separated format descriptions --- 'f8, i4, a5'
+        list of format description strings --- ['f8', 'i4', 'a5']
+    names : string or (list or tuple of strings)
+        comma-separated field names --- 'col1, col2, col3'
+        list or tuple of field names
+    titles : sequence
+        sequence of title strings or unicode
+    aligned : bool
+        align the fields by padding as the C-compiler would
+    byteorder :
+        If specified, all the fields will be changed to the
+        provided byteorder.  Otherwise, the default byteorder is
+        used.
+
+    Returns
+    -------
+    object
+        A Python object whose dtype attribute is a data-type.
+    """
     def __init__(self, formats, names, titles, aligned=False, byteorder=None):
         self._parseFormats(formats, aligned)
         self._setfieldnames(names, titles)
         self._createdescr(byteorder)
+        self.dtype = self._descr
 
     def _parseFormats(self, formats, aligned=0):
         """ Parse the field formats """
@@ -115,6 +181,8 @@ class format_parser:
         self._descr = descr
 
 class record(nt.void):
+    """A data-type scalar that allows field access as attribute lookup.
+    """
     def __repr__(self):
         return self.__str__()
 
@@ -182,6 +250,109 @@ class record(nt.void):
 #  the fields (and any subfields)
 
 class recarray(ndarray):
+    """
+    Construct an ndarray that allows field access using attributes.
+
+    Arrays may have a data-types containing fields, analagous
+    to columns in a spread sheet.  An example is ``[(x, int), (y, float)]``,
+    where each entry in the array is a pair of ``(int, float)``.  Normally,
+    these attributes are accessed using dictionary lookups such as ``arr['x']``
+    and ``arr['y']``.  Record arrays allow the fields to be accessed as members
+    of the array, using ``arr.x`` and ``arr.y``.
+
+    Parameters
+    ----------
+    shape : tuple
+        Shape of output array.
+    dtype : data-type, optional
+        The desired data-type.  By default, the data-type is determined
+        from `formats`, `names`, `titles`, `aligned` and `byteorder`.
+    formats : list of data-types, optional
+        A list containing the data-types for the different columns, e.g.
+        ``['i4', 'f8', 'i4']``.  `formats` does *not* support the new
+        convention of using types directly, i.e. ``(int, float, int)``.
+        Note that `formats` must be a list, not a tuple.
+        Given that `formats` is somewhat limited, we recommend specifying
+        `dtype` instead.
+    names : tuple of strings, optional
+        The name of each column, e.g. ``('x', 'y', 'z')``.
+    buf : buffer, optional
+        By default, a new array is created of the given shape and data-type.
+        If `buf` is specified and is an object exposing the buffer interface,
+        the array will use the memory from the existing buffer.  In this case,
+        the `offset` and `strides` keywords are available.
+
+    Other Parameters
+    ----------------
+    titles : tuple of strings, optional
+        Aliases for column names.  For example, if `names` were
+        ``('x', 'y', 'z')`` and `titles` is
+        ``('x_coordinate', 'y_coordinate', 'z_coordinate')``, then
+        ``arr['x']`` is equivalent to both ``arr.x`` and ``arr.x_coordinate``.
+    byteorder : {'<', '>', '='}, optional
+        Byte-order for all fields.
+    aligned : {True, False}, optional
+        Align the fields in memory as the C-compiler would.
+    strides : tuple of ints, optional
+        Buffer (`buf`) is interpreted according to these strides (strides
+        define how many bytes each array element, row, column, etc.
+        occupy in memory).
+    offset : int, optional
+        Start reading buffer (`buf`) from this offset onwards.
+
+    Returns
+    -------
+    rec : recarray
+        Empty array of the given shape and type.
+
+    See Also
+    --------
+    rec.fromrecords : Construct a record array from data.
+    record : fundamental data-type for recarray
+    format_parser : determine a data-type from formats, names, titles
+
+    Notes
+    -----
+    This constructor can be compared to ``empty``: it creates a new record
+    array but does not fill it with data.  To create a reccord array from data,
+    use one of the following methods:
+
+    1. Create a standard ndarray and convert it to a record array,
+       using ``arr.view(np.recarray)``
+    2. Use the `buf` keyword.
+    3. Use `np.rec.fromrecords`.
+
+    Examples
+    --------
+    Create an array with two fields, ``x`` and ``y``:
+
+    >>> x = np.array([(1.0, 2), (3.0, 4)], dtype=[('x', float), ('y', int)])
+    >>> x
+    array([(1.0, 2), (3.0, 4)],
+          dtype=[('x', '<f8'), ('y', '<i4')])
+
+    >>> x['x']
+    array([ 1.,  3.])
+
+    View the array as a record array:
+
+    >>> x = x.view(np.recarray)
+
+    >>> x.x
+    array([ 1.,  3.])
+
+    >>> x.y
+    array([2, 4])
+
+    Create a new, empty record array:
+
+    >>> np.recarray((2,),
+    ... dtype=[('x', int), ('y', float), ('z', int)]) #doctest: +SKIP
+    rec.array([(-1073741821, 1.2249118382103472e-301, 24547520),
+           (3471280, 1.2134086255804012e-316, 0)],
+          dtype=[('x', '<i4'), ('y', '<f8'), ('z', '<i4')])
+
+    """
     def __new__(subtype, shape, dtype=None, buf=None, offset=0, strides=None,
                 formats=None, names=None, titles=None,
                 byteorder=None, aligned=False):
@@ -254,6 +425,10 @@ class recarray(ndarray):
             return obj.view(ndarray)
         return obj
 
+    def __repr__(self) :
+        ret = ndarray.__repr__(self)
+        return ret.replace("recarray", "rec.array", 1)
+
     def field(self, attr, val=None):
         if isinstance(attr, int):
             names = ndarray.__getattribute__(self,'dtype').names
@@ -273,25 +448,31 @@ class recarray(ndarray):
         else:
             return self.setfield(val, *res)
 
-    def view(self, obj):
-        try:
-            if issubclass(obj, ndarray):
-                return ndarray.view(self, obj)
-        except TypeError:
-            pass
-        dtype = sb.dtype(obj)
-        if dtype.fields is None:
-            return self.__array__().view(dtype)
-        return ndarray.view(self, obj)
+    def view(self, dtype=None, type=None):
+        if dtype is None:
+            return ndarray.view(self, type)
+        elif type is None:
+            try:
+                if issubclass(dtype, ndarray):
+                    return ndarray.view(self, dtype)
+            except TypeError:
+                pass
+            dtype = sb.dtype(dtype)
+            if dtype.fields is None:
+                return self.__array__().view(dtype)
+            return ndarray.view(self, dtype)
+        else:
+            return ndarray.view(self, dtype, type)
+
 
 def fromarrays(arrayList, dtype=None, shape=None, formats=None,
                names=None, titles=None, aligned=False, byteorder=None):
     """ create a record array from a (flat) list of arrays
 
-    >>> x1=N.array([1,2,3,4])
-    >>> x2=N.array(['a','dd','xyz','12'])
-    >>> x3=N.array([1.1,2,3,4])
-    >>> r = fromarrays([x1,x2,x3],names='a,b,c')
+    >>> x1=np.array([1,2,3,4])
+    >>> x2=np.array(['a','dd','xyz','12'])
+    >>> x3=np.array([1.1,2,3,4])
+    >>> r = np.core.records.fromarrays([x1,x2,x3],names='a,b,c')
     >>> print r[1]
     (2, 'dd', 2.0)
     >>> x1[1]=34
@@ -369,7 +550,8 @@ def fromrecords(recList, dtype=None, shape=None, formats=None, names=None,
         If formats is None, then this will auto-detect formats. Use list of
         tuples rather than list of lists for faster processing.
 
-    >>> r=fromrecords([(456,'dbe',1.2),(2,'de',1.3)],names='col1,col2,col3')
+    >>> r=np.core.records.fromrecords([(456,'dbe',1.2),(2,'de',1.3)],
+    ... names='col1,col2,col3')
     >>> print r[0]
     (456, 'dbe', 1.2)
     >>> r.col1
@@ -455,7 +637,7 @@ def fromfile(fd, dtype=None, shape=None, offset=0, formats=None,
     to be a file object.
 
     >>> from tempfile import TemporaryFile
-    >>> a = N.empty(10,dtype='f8,i4,a5')
+    >>> a = np.empty(10,dtype='f8,i4,a5')
     >>> a[5] = (0.5,10,'abcde')
     >>>
     >>> fd=TemporaryFile()
@@ -463,7 +645,8 @@ def fromfile(fd, dtype=None, shape=None, offset=0, formats=None,
     >>> a.tofile(fd)
     >>>
     >>> fd.seek(0)
-    >>> r=fromfile(fd, formats='f8,i4,a5', shape=10, byteorder='<')
+    >>> r=np.core.records.fromfile(fd, formats='f8,i4,a5', shape=10,
+    ... byteorder='<')
     >>> print r[5]
     (0.5, 10, 'abcde')
     >>> r.shape
