@@ -299,7 +299,7 @@ def CCompiler_cxx_compiler(self):
     cxx.compiler_so = [cxx.compiler_cxx[0]] + cxx.compiler_so[1:]
     if sys.platform.startswith('aix') and 'ld_so_aix' in cxx.linker_so[0]:
         # AIX needs the ld_so_aix script included with Python
-        cxx.linker_so = [cxx.linker_so[0]] + cxx.compiler_cxx[0] \
+        cxx.linker_so = [cxx.linker_so[0], cxx.compiler_cxx[0]] \
                         + cxx.linker_so[2:]
     else:
         cxx.linker_so = [cxx.compiler_cxx[0]] + cxx.linker_so[1:]
@@ -390,8 +390,7 @@ ccompiler.gen_lib_options = gen_lib_options
 for _cc in ['msvc', 'bcpp', 'cygwinc', 'emxc', 'unixc']:
     _m = sys.modules.get('distutils.'+_cc+'compiler')
     if _m is not None:
-        setattr(getattr(_m, _cc+'compiler'), 'gen_lib_options',
-                gen_lib_options)
+        setattr(_m, 'gen_lib_options', gen_lib_options)
 
 _distutils_gen_preprocess_options = gen_preprocess_options
 def gen_preprocess_options (macros, include_dirs):
@@ -400,6 +399,10 @@ def gen_preprocess_options (macros, include_dirs):
 ccompiler.gen_preprocess_options = gen_preprocess_options
 
 ##Fix distutils.util.split_quoted:
+# NOTE:  I removed this fix in revision 4481 (see ticket #619), but it appears
+# that removing this fix causes f2py problems on Windows XP (see ticket #723).
+# Specifically, on WinXP when gfortran is installed in a directory path, which
+# contains spaces, then f2py is unable to find it.
 import re
 import string
 _wordchars_re = re.compile(r'[^\\\'\"%s ]*' % string.whitespace)
@@ -456,6 +459,7 @@ def split_quoted(s):
 
     return words
 ccompiler.split_quoted = split_quoted
+##Fix distutils.util.split_quoted:
 
 # define DISTUTILS_USE_SDK when necessary to workaround distutils/msvccompiler.py bug
 msvc_on_amd64()
