@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/bin/env python
 """
 This file defines a set of system_info classes for getting
 information about various resources (libraries, library directories,
@@ -58,24 +58,24 @@ Global parameters:
 The file 'site.cfg' is looked for in
 
 1) Directory of main setup.py file being run.
-2) Home directory of user running the setup.py file (Not implemented yet)
+2) Home directory of user running the setup.py file as ~/.numpy-site.cfg
 3) System wide directory (location of this file...)
 
 The first one found is used to get system configuration options The
 format is that used by ConfigParser (i.e., Windows .INI style). The
-section DEFAULT has options that are the default for each section. The
+section ALL has options that are the default for each section. The
 available sections are fftw, atlas, and x11. Appropiate defaults are
 used if nothing is specified.
 
 The order of finding the locations of resources is the following:
  1. environment variable
  2. section in site.cfg
- 3. DEFAULT section in site.cfg
+ 3. ALL section in site.cfg
 Only the first complete match is returned.
 
 Example:
 ----------
-[DEFAULT]
+[ALL]
 library_dirs = /usr/lib:/usr/local/lib:/opt/lib
 include_dirs = /usr/include:/usr/local/include:/opt/include
 src_dirs = /usr/local/src:/opt/src
@@ -104,22 +104,73 @@ Authors:
 Copyright 2002 Pearu Peterson all rights reserved,
 Pearu Peterson <pearu@cens.ioc.ee>
 Permission to use, modify, and distribute this software is given under the
-terms of the SciPy (BSD style) license.  See LICENSE.txt that came with
+terms of the NumPy (BSD style) license.  See LICENSE.txt that came with
 this distribution for specifics.
 
 NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
 """
 
-import sys,os,re,copy
+import sys
+import os
+import re
+import copy
 import warnings
-from distutils.errors import DistutilsError
 from glob import glob
 import ConfigParser
-from exec_command import find_executable, exec_command, get_pythonexe
-from numpy.distutils.misc_util import is_sequence, is_string
-import distutils.sysconfig
 
-from distutils.sysconfig import get_config_vars
+from distutils.errors import DistutilsError
+from distutils.dist import Distribution
+import distutils.sysconfig
+from distutils import log
+
+from numpy.distutils.exec_command import \
+    find_executable, exec_command, get_pythonexe
+from numpy.distutils.misc_util import is_sequence, is_string
+from numpy.distutils.command.config import config as cmd_config
+
+# Determine number of bits
+import platform
+_bits = {'32bit':32,'64bit':64}
+platform_bits = _bits[platform.architecture()[0]]
+
+def libpaths(paths,bits):
+    """Return a list of library paths valid on 32 or 64 bit systems.
+
+    Inputs:
+      paths : sequence
+        A sequence of strings (typically paths)
+      bits : int
+        An integer, the only valid values are 32 or 64.  A ValueError exception
+      is raised otherwise.
+
+    Examples:
+
+    Consider a list of directories
+    >>> paths = ['/usr/X11R6/lib','/usr/X11/lib','/usr/lib']
+
+    For a 32-bit platform, this is already valid:
+    >>> libpaths(paths,32)
+    ['/usr/X11R6/lib', '/usr/X11/lib', '/usr/lib']
+
+    On 64 bits, we prepend the '64' postfix
+    >>> libpaths(paths,64)
+    ['/usr/X11R6/lib64', '/usr/X11R6/lib', '/usr/X11/lib64', '/usr/X11/lib',
+    '/usr/lib64', '/usr/lib']
+    """
+    if bits not in (32, 64):
+        raise ValueError("Invalid bit size in libpaths: 32 or 64 only")
+
+    # Handle 32bit case
+    if bits==32:
+        return paths
+
+    # Handle 64bit case
+    out = []
+    for p in paths:
+        out.extend([p+'64', p])
+
+    return out
+
 
 if sys.platform == 'win32':
     default_lib_dirs = ['C:\\',
@@ -130,13 +181,16 @@ if sys.platform == 'win32':
     default_x11_lib_dirs = []
     default_x11_include_dirs = []
 else:
-    default_lib_dirs = ['/usr/local/lib', '/opt/lib', '/usr/lib',
-                        '/opt/local/lib', '/sw/lib']
+    default_lib_dirs = libpaths(['/usr/local/lib','/opt/lib','/usr/lib',
+                                 '/opt/local/lib','/sw/lib'], platform_bits)
     default_include_dirs = ['/usr/local/include',
                             '/opt/include', '/usr/include',
-                            '/opt/local/include', '/sw/include']
+                            '/opt/local/include', '/sw/include',
+                            '/usr/include/suitesparse']
     default_src_dirs = ['.','/usr/local/src', '/opt/src','/sw/src']
-    default_x11_lib_dirs = ['/usr/X11R6/lib','/usr/X11/lib','/usr/lib']
+
+    default_x11_lib_dirs = libpaths(['/usr/X11R6/lib','/usr/X11/lib',
+                                     '/usr/lib'], platform_bits)
     default_x11_include_dirs = ['/usr/X11R6/include','/usr/X11/include',
                                 '/usr/include']
 
@@ -149,7 +203,7 @@ default_lib_dirs = filter(os.path.isdir, default_lib_dirs)
 default_include_dirs = filter(os.path.isdir, default_include_dirs)
 default_src_dirs = filter(os.path.isdir, default_src_dirs)
 
-so_ext = get_config_vars('SO')[0] or ''
+so_ext = distutils.sysconfig.get_config_vars('SO')[0] or ''
 
 def get_standard_file(fname):
     """Returns a list of files named 'fname' from
@@ -319,7 +373,7 @@ class system_info:
 
     """ get_info() is the only public method. Don't use others.
     """
-    section = 'DEFAULT'
+    section = 'ALL'
     dir_env_var = None
     search_static_first = 0 # XXX: disabled by default, may disappear in
                             # future unless it is proved to be useful.
@@ -342,16 +396,20 @@ class system_info:
         defaults['src_dirs'] = os.pathsep.join(default_src_dirs)
         defaults['search_static_first'] = str(self.search_static_first)
         self.cp = ConfigParser.ConfigParser(defaults)
-        self.files = get_standard_file('site.cfg')
+        self.files = []
+        self.files.extend(get_standard_file('.numpy-site.cfg'))
+        self.files.extend(get_standard_file('site.cfg'))
         self.parse_config_files()
-        self.search_static_first = self.cp.getboolean(self.section,
-                                                      'search_static_first')
+        if self.section is not None:
+            self.search_static_first = self.cp.getboolean(self.section,
+                                                          'search_static_first')
         assert isinstance(self.search_static_first, int)
 
     def parse_config_files(self):
         self.cp.read(self.files)
         if not self.cp.has_section(self.section):
-            self.cp.add_section(self.section)
+            if self.section is not None:
+                self.cp.add_section(self.section)
 
     def calc_libraries_info(self):
         libs = self.get_libraries()
@@ -366,7 +424,7 @@ class system_info:
             if i is not None:
                 dict_append(info,**i)
             else:
-                print 'Library %s was not found. Ignoring' % (lib)
+                log.info('Library %s was not found. Ignoring' % (lib))
         return info
 
     def set_info(self,**info):
@@ -376,7 +434,7 @@ class system_info:
         self.saved_results[self.__class__.__name__] = info
 
     def has_info(self):
-        return self.saved_results.has_key(self.__class__.__name__)
+        return self.__class__.__name__ in self.saved_results
 
     def get_info(self,notfound_action=0):
         """ Return a dictonary with items that are compatible
@@ -385,8 +443,7 @@ class system_info:
         flag = 0
         if not self.has_info():
             flag = 1
-            if self.verbosity>0:
-                print self.__class__.__name__ + ':'
+            log.info(self.__class__.__name__ + ':')
             if hasattr(self, 'calc_info'):
                 self.calc_info()
             if notfound_action:
@@ -398,20 +455,20 @@ class system_info:
                     else:
                         raise ValueError(repr(notfound_action))
 
-            if self.verbosity>0:
-                if not self.has_info():
-                    print '  NOT AVAILABLE'
-                    self.set_info()
-                else:
-                    print '  FOUND:'
+            if not self.has_info():
+                log.info('  NOT AVAILABLE')
+                self.set_info()
+            else:
+                log.info('  FOUND:')
 
         res = self.saved_results.get(self.__class__.__name__)
         if self.verbosity>0 and flag:
             for k,v in res.items():
                 v = str(v)
-                if k=='sources' and len(v)>200: v = v[:60]+' ...\n... '+v[-60:]
-                print '    %s = %s'%(k,v)
-            print
+                if k in ['sources','libraries'] and len(v)>270:
+                    v = v[:120]+'...\n...\n...'+v[-120:]
+                log.info('    %s = %s', k, v)
+            log.info('')
 
         return copy.deepcopy(res)
 
@@ -422,17 +479,17 @@ class system_info:
             if is_sequence(env_var):
                 e0 = env_var[-1]
                 for e in env_var:
-                    if os.environ.has_key(e):
+                    if e in os.environ:
                         e0 = e
                         break
                 if not env_var[0]==e0:
-                    print 'Setting %s=%s' % (env_var[0],e0)
+                    log.info('Setting %s=%s' % (env_var[0],e0))
                 env_var = e0
-        if env_var and os.environ.has_key(env_var):
+        if env_var and env_var in os.environ:
             d = os.environ[env_var]
             if d=='None':
-                print 'Disabled',self.__class__.__name__,'(%s is None)' \
-                      % (self.dir_env_var)
+                log.info('Disabled %s: %s',self.__class__.__name__,'(%s is None)' \
+                      % (env_var,))
                 return []
             if os.path.isfile(d):
                 dirs = [os.path.dirname(d)] + dirs
@@ -441,8 +498,8 @@ class system_info:
                     b = os.path.basename(d)
                     b = os.path.splitext(b)[0]
                     if b[:3]=='lib':
-                        print 'Replacing _lib_names[0]==%r with %r' \
-                              % (self._lib_names[0], b[3:])
+                        log.info('Replacing _lib_names[0]==%r with %r' \
+                              % (self._lib_names[0], b[3:]))
                         self._lib_names[0] = b[3:]
             else:
                 ds = d.split(os.pathsep)
@@ -455,12 +512,13 @@ class system_info:
                             if os.path.isdir(d1):
                                 ds2.append(d1)
                 dirs = ds2 + dirs
-        default_dirs = self.cp.get('DEFAULT', key).split(os.pathsep)
+        default_dirs = self.cp.get(self.section, key).split(os.pathsep)
         dirs.extend(default_dirs)
         ret = []
-        [ret.append(d) for d in dirs if os.path.isdir(d) and d not in ret]
-        if self.verbosity>1:
-            print '(',key,'=',':'.join(ret),')'
+        for d in dirs:
+            if os.path.isdir(d) and d not in ret:
+                ret.append(d)
+        log.debug('( %s = %s )', key, ':'.join(ret))
         return ret
 
     def get_lib_dirs(self, key='library_dirs'):
@@ -486,64 +544,94 @@ class system_info:
     def get_libraries(self, key='libraries'):
         return self.get_libs(key,'')
 
-    def check_libs(self,lib_dir,libs,opt_libs =[]):
-        """ If static or shared libraries are available then return
-            their info dictionary. """
+    def library_extensions(self):
+        static_exts = ['.a']
+        if sys.platform == 'win32':
+            static_exts.append('.lib')  # .lib is used by MSVC
         if self.search_static_first:
-            exts = ['.a',so_ext]
+            exts = static_exts + [so_ext]
         else:
-            exts = [so_ext,'.a']
-        if sys.platform=='cygwin':
+            exts = [so_ext] + static_exts
+        if sys.platform == 'cygwin':
             exts.append('.dll.a')
+        if sys.platform == 'darwin':
+            exts.append('.dylib')
+        # Debian and Ubuntu added a g3f suffix to shared library to deal with
+        # g77 -> gfortran ABI transition
+        # XXX: disabled, it hides more problem than it solves.
+        #if sys.platform[:5] == 'linux':
+        #    exts.append('.so.3gf')
+        return exts
+
+    def check_libs(self,lib_dir,libs,opt_libs =[]):
+        """If static or shared libraries are available then return
+        their info dictionary.
+
+        Checks for all libraries as shared libraries first, then
+        static (or vice versa if self.search_static_first is True).
+        """
+        exts = self.library_extensions()
+        info = None
         for ext in exts:
             info = self._check_libs(lib_dir,libs,opt_libs,[ext])
-            if info is not None: return info
-        return
+            if info is not None:
+                break
+        if not info:
+            log.info('  libraries %s not found in %s', ','.join(libs), lib_dir)
+        return info
 
-    def check_libs2(self,lib_dir,libs,opt_libs =[]):
-        """ If static or shared libraries are available then return
-            their info dictionary. """
-        if self.search_static_first:
-            exts = ['.a',so_ext]
-        else:
-            exts = [so_ext,'.a']
-        if sys.platform=='cygwin':
-            exts.append('.dll.a')
+    def check_libs2(self, lib_dir, libs, opt_libs =[]):
+        """If static or shared libraries are available then return
+        their info dictionary.
+
+        Checks each library for shared or static.
+        """
+        exts = self.library_extensions()
         info = self._check_libs(lib_dir,libs,opt_libs,exts)
-        if info is not None: return info
-        return
+        if not info:
+            log.info('  libraries %s not found in %s', ','.join(libs), lib_dir)
+        return info
 
     def _lib_list(self, lib_dir, libs, exts):
         assert is_string(lib_dir)
         liblist = []
+        # under windows first try without 'lib' prefix
+        if sys.platform == 'win32':
+            lib_prefixes = ['', 'lib']
+        else:
+            lib_prefixes = ['lib']
+        # for each library name, see if we can find a file for it.
         for l in libs:
             for ext in exts:
-                p = self.combine_paths(lib_dir, 'lib'+l+ext)
+                for prefix in lib_prefixes:
+                    p = self.combine_paths(lib_dir, prefix+l+ext)
+                    if p:
+                        break
                 if p:
                     assert len(p)==1
-                    liblist.append(p[0])
+                    # ??? splitext on p[0] would do this for cygwin
+                    # doesn't seem correct
+                    if ext == '.dll.a':
+                        l += '.dll'
+                    liblist.append(l)
                     break
         return liblist
 
-    def _extract_lib_names(self,libs):
-        return [os.path.splitext(os.path.basename(p))[0][3:] \
-                for p in libs]
-
-    def _check_libs(self,lib_dir,libs, opt_libs, exts):
+    def _check_libs(self, lib_dir, libs, opt_libs, exts):
         found_libs = self._lib_list(lib_dir, libs, exts)
         if len(found_libs) == len(libs):
-            found_libs = self._extract_lib_names(found_libs)
             info = {'libraries' : found_libs, 'library_dirs' : [lib_dir]}
             opt_found_libs = self._lib_list(lib_dir, opt_libs, exts)
             if len(opt_found_libs) == len(opt_libs):
-                opt_found_libs = self._extract_lib_names(opt_found_libs)
                 info['libraries'].extend(opt_found_libs)
             return info
         else:
-            print '  looking libraries %s in %s but found %s' % \
-                  (','.join(libs), lib_dir, ','.join(found_libs) or None)
+            return None
 
     def combine_paths(self,*args):
+        """Return a list of existing paths composed by all combinations
+        of items from the arguments.
+        """
         return combine_paths(*args,**{'verbosity':self.verbosity})
 
 
@@ -607,8 +695,7 @@ class fftw_info(system_info):
             self.set_info(**info)
             return True
         else:
-            if self.verbosity>0:
-                print '  %s not found' % (ver_param['name'])
+            log.info('  %s not found' % (ver_param['name']))
             return False
 
     def calc_info(self):
@@ -778,7 +865,13 @@ class mkl_info(system_info):
             return
         info = {}
         dict_append(info,**mkl)
-        dict_append(info,libraries = ['pthread'], include_dirs = incl_dirs)
+        dict_append(info,
+                    define_macros=[('SCIPY_MKL_H',None)],
+                    include_dirs = incl_dirs)
+        if sys.platform == 'win32':
+            pass # win32 has no pthread library
+        else:
+            dict_append(info, libraries=['pthread'])
         self.set_info(**info)
 
 class lapack_mkl_info(mkl_info):
@@ -787,7 +880,11 @@ class lapack_mkl_info(mkl_info):
         mkl = get_info('mkl')
         if not mkl:
             return
-        lapack_libs = self.get_libs('lapack_libs',['mkl_lapack32','mkl_lapack64'])
+        if sys.platform == 'win32':
+            lapack_libs = self.get_libs('lapack_libs',['mkl_lapack'])
+        else:
+            lapack_libs = self.get_libs('lapack_libs',['mkl_lapack32','mkl_lapack64'])
+
         info = {'libraries': lapack_libs}
         dict_append(info,**mkl)
         self.set_info(**info)
@@ -826,12 +923,12 @@ class atlas_info(system_info):
         lapack = None
         atlas_1 = None
         for d in lib_dirs:
-            atlas = self.check_libs(d,atlas_libs,[])
-            lapack_atlas = self.check_libs(d,['lapack_atlas'],[])
+            atlas = self.check_libs2(d,atlas_libs,[])
+            lapack_atlas = self.check_libs2(d,['lapack_atlas'],[])
             if atlas is not None:
-                lib_dirs2 = self.combine_paths(d,['atlas*','ATLAS*'])+[d]
+                lib_dirs2 = [d] + self.combine_paths(d,['atlas*','ATLAS*'])
                 for d2 in lib_dirs2:
-                    lapack = self.check_libs(d2,lapack_libs,[])
+                    lapack = self.check_libs2(d2,lapack_libs,[])
                     if lapack is not None:
                         break
                 else:
@@ -840,7 +937,7 @@ class atlas_info(system_info):
                     break
             if atlas:
                 atlas_1 = atlas
-        print self.__class__
+        log.info(self.__class__)
         if atlas is None:
             atlas = atlas_1
         if atlas is None:
@@ -875,10 +972,16 @@ class atlas_info(system_info):
         lapack_dir = lapack['library_dirs'][0]
         lapack_name = lapack['libraries'][0]
         lapack_lib = None
-        for e in ['.a',so_ext]:
-            fn = os.path.join(lapack_dir,'lib'+lapack_name+e)
-            if os.path.exists(fn):
-                lapack_lib = fn
+        lib_prefixes = ['lib']
+        if sys.platform == 'win32':
+            lib_prefixes.append('')
+        for e in self.library_extensions():
+            for prefix in lib_prefixes:
+                fn = os.path.join(lapack_dir,prefix+lapack_name+e)
+                if os.path.exists(fn):
+                    lapack_lib = fn
+                    break
+            if lapack_lib:
                 break
         if lapack_lib is not None:
             sz = os.stat(lapack_lib)[6]
@@ -908,7 +1011,7 @@ class atlas_blas_info(atlas_info):
                                    self._lib_names + self._lib_atlas)
         atlas = None
         for d in lib_dirs:
-            atlas = self.check_libs(d,atlas_libs,[])
+            atlas = self.check_libs2(d,atlas_libs,[])
             if atlas is not None:
                 break
         if atlas is None:
@@ -982,9 +1085,12 @@ class lapack_src_info(system_info):
         if not src_dir:
             #XXX: Get sources from netlib. May be ask first.
             return
-        # The following is extracted from LAPACK-3.0/SRC/Makefile
+        # The following is extracted from LAPACK-3.0/SRC/Makefile.
+        # Added missing names from lapack-lite-3.1.1/SRC/Makefile
+        # while keeping removed names for Lapack-3.0 compatibility.
         allaux='''
         ilaenv ieeeck lsame lsamen xerbla
+        iparmq
         ''' # *.f
         laux = '''
         bdsdc bdsqr disna labad lacpy ladiv lae2 laebz laed0 laed1
@@ -994,6 +1100,9 @@ class lapack_src_info(system_info):
         lasd5 lasd6 lasd7 lasd8 lasd9 lasda lasdq lasdt laset lasq1
         lasq2 lasq3 lasq4 lasq5 lasq6 lasr lasrt lassq lasv2 pttrf
         stebz stedc steqr sterf
+
+        larra larrc larrd larr larrk larrj larrr laneg laisnan isnan
+        lazq3 lazq4
         ''' # [s|d]*.f
         lasrc = '''
         gbbrd gbcon gbequ gbrfs gbsv gbsvx gbtf2 gbtrf gbtrs gebak
@@ -1017,6 +1126,8 @@ class lapack_src_info(system_info):
         tgexc tgsen tgsja tgsna tgsy2 tgsyl tpcon tprfs tptri tptrs
         trcon trevc trexc trrfs trsen trsna trsyl trti2 trtri trtrs
         tzrqf tzrzf
+
+        lacn2 lahr2 stemr laqr0 laqr1 laqr2 laqr3 laqr4 laqr5
         ''' # [s|c|d|z]*.f
         sd_lasrc = '''
         laexc lag2 lagv2 laln2 lanv2 laqtr lasy2 opgtr opmtr org2l
@@ -1054,85 +1165,51 @@ class lapack_src_info(system_info):
                   + ['z%s.f'%f for f in (zlasrc).split()] \
                   + ['%s.f'%f for f in (allaux+oclasrc+ozlasrc).split()]
         sources = [os.path.join(src_dir,f) for f in sources]
-        #XXX: should we check here actual existence of source files?
+        # Lapack 3.1:
+        src_dir2 = os.path.join(src_dir,'..','INSTALL')
+        sources += [os.path.join(src_dir2,p+'lamch.f') for p in 'sdcz']
+        # Should we check here actual existence of source files?
+        # Yes, the file listing is different between 3.0 and 3.1
+        # versions.
+        sources = [f for f in sources if os.path.isfile(f)]
         info = {'sources':sources,'language':'f77'}
         self.set_info(**info)
 
 atlas_version_c_text = r'''
-/* This file is generated from numpy_distutils/system_info.py */
-#ifdef __CPLUSPLUS__
-extern "C" {
-#endif
-#include "Python.h"
-static PyMethodDef module_methods[] = { {NULL,NULL} };
-PyMODINIT_FUNC initatlas_version(void) {
-  void ATL_buildinfo(void);
+/* This file is generated from numpy/distutils/system_info.py */
+void ATL_buildinfo(void);
+int main(void) {
   ATL_buildinfo();
-  Py_InitModule("atlas_version", module_methods);
+  return 0;
 }
-#ifdef __CPLUSCPLUS__
-}
-#endif
 '''
 
-def _get_build_temp():
-    from distutils.util import get_platform
-    plat_specifier = ".%s-%s" % (get_platform(), sys.version[0:3])
-    return os.path.join('build','temp'+plat_specifier)
-
+_cached_atlas_version = {}
 def get_atlas_version(**config):
-    os.environ['NO_SCIPY_IMPORT']='get_atlas_version'
-    from core import Extension, setup
-    from misc_util import get_cmd
-    import log
-    magic = hex(hash(repr(config)))
-    def atlas_version_c(extension, build_dir,magic=magic):
-        source = os.path.join(build_dir,'atlas_version_%s.c' % (magic))
-        if os.path.isfile(source):
-            from distutils.dep_util import newer
-            if newer(source,__file__):
-                return source
-        f = open(source,'w')
-        f.write(atlas_version_c_text)
-        f.close()
-        return source
-    ext = Extension('atlas_version',
-                    sources=[atlas_version_c],
-                    **config)
-    build_dir = _get_build_temp()
-    extra_args = ['--build-lib',build_dir]
-    for a in sys.argv:
-        if re.match('[-][-]compiler[=]',a):
-            extra_args.append(a)
-    import distutils.core
-    old_dist = distutils.core._setup_distribution
-    distutils.core._setup_distribution = None
-    return_flag = True
-    try:
-        dist = setup(ext_modules=[ext],
-                     script_name = 'get_atlas_version',
-                     script_args = ['build_src','build_ext']+extra_args)
-        return_flag = False
-    except Exception,msg:
-        print "##### msg: %s" % msg
-        if not msg:
-            msg = "Unknown Exception"
-        log.warn(msg)
-    distutils.core._setup_distribution = old_dist
-
-    if return_flag:
-        return
-
-    from distutils.sysconfig import get_config_var
-    so_ext = get_config_var('SO')
-    target = os.path.join(build_dir,'atlas_version'+so_ext)
-    cmd = [get_pythonexe(),'-c',
-           '"import imp,os;os.environ[\\"NO_SCIPY_IMPORT\\"]='\
-           '\\"system_info.get_atlas_version:load atlas_version\\";'\
-           'imp.load_dynamic(\\"atlas_version\\",\\"%s\\")"'\
-           % (os.path.basename(target))]
-    s,o = exec_command(cmd,execute_in=os.path.dirname(target),use_tee=0)
+    libraries = config.get('libraries', [])
+    library_dirs = config.get('library_dirs', [])
+    key = (tuple(libraries), tuple(library_dirs))
+    if key in _cached_atlas_version:
+        return _cached_atlas_version[key]
+    c = cmd_config(Distribution())
     atlas_version = None
+    try:
+        s, o = c.get_output(atlas_version_c_text,
+                            libraries=libraries, library_dirs=library_dirs)
+    except: # failed to get version from file -- maybe on Windows
+        # look at directory name
+        for o in library_dirs:
+            m = re.search(r'ATLAS_(?P<version>\d+[.]\d+[.]\d+)_',o)
+            if m:
+                atlas_version = m.group('version')
+            if atlas_version is not None:
+                break
+        # final choice --- look at ATLAS_VERSION environment
+        #   variable
+        if atlas_version is None:
+            atlas_version = os.environ.get('ATLAS_VERSION',None)
+        return atlas_version or '?.?.?'
+
     if not s:
         m = re.search(r'ATLAS version (?P<version>\d+[.]\d+[.]\d+)',o)
         if m:
@@ -1141,24 +1218,37 @@ def get_atlas_version(**config):
         if re.search(r'undefined symbol: ATL_buildinfo',o,re.M):
             atlas_version = '3.2.1_pre3.3.6'
         else:
-            print 'Command:',' '.join(cmd)
-            print 'Status:',s
-            print 'Output:',o
+            log.info('Status: %d', s)
+            log.info('Output: %s', o)
+    _cached_atlas_version[key] = atlas_version
     return atlas_version
 
+from distutils.util import get_platform
 
 class lapack_opt_info(system_info):
 
+    notfounderror = LapackNotFoundError
+
     def calc_info(self):
 
         if sys.platform=='darwin' and not os.environ.get('ATLAS',None):
             args = []
             link_args = []
+            if get_platform()[-4:] == 'i386':
+                intel = 1
+            else:
+                intel = 0
             if os.path.exists('/System/Library/Frameworks/Accelerate.framework/'):
-                args.extend(['-faltivec'])
+                if intel:
+                    args.extend(['-msse3'])
+                else:
+                    args.extend(['-faltivec'])
                 link_args.extend(['-Wl,-framework','-Wl,Accelerate'])
             elif os.path.exists('/System/Library/Frameworks/vecLib.framework/'):
-                args.extend(['-faltivec'])
+                if intel:
+                    args.extend(['-msse3'])
+                else:
+                    args.extend(['-faltivec'])
                 link_args.extend(['-Wl,-framework','-Wl,vecLib'])
             if args:
                 self.set_info(extra_compile_args=args,
@@ -1182,7 +1272,7 @@ class lapack_opt_info(system_info):
         if atlas_info:
             version_info = atlas_info.copy()
             atlas_version = get_atlas_version(**version_info)
-            if not atlas_info.has_key('define_macros'):
+            if 'define_macros' not in atlas_info:
                 atlas_info['define_macros'] = []
             if atlas_version is None:
                 atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
@@ -1234,20 +1324,32 @@ class lapack_opt_info(system_info):
 
 class blas_opt_info(system_info):
 
+    notfounderror = BlasNotFoundError
+
     def calc_info(self):
 
         if sys.platform=='darwin' and not os.environ.get('ATLAS',None):
             args = []
             link_args = []
+            if get_platform()[-4:] == 'i386':
+                intel = 1
+            else:
+                intel = 0
             if os.path.exists('/System/Library/Frameworks/Accelerate.framework/'):
-                args.extend(['-faltivec',
-                    '-I/System/Library/Frameworks/vecLib.framework/Headers',
-                    ])
+                if intel:
+                    args.extend(['-msse3'])
+                else:
+                    args.extend(['-faltivec'])
+                args.extend([
+                    '-I/System/Library/Frameworks/vecLib.framework/Headers'])
                 link_args.extend(['-Wl,-framework','-Wl,Accelerate'])
             elif os.path.exists('/System/Library/Frameworks/vecLib.framework/'):
-                args.extend(['-faltivec',
-                    '-I/System/Library/Frameworks/vecLib.framework/Headers',
-                    ])
+                if intel:
+                    args.extend(['-msse3'])
+                else:
+                    args.extend(['-faltivec'])
+                args.extend([
+                    '-I/System/Library/Frameworks/vecLib.framework/Headers'])
                 link_args.extend(['-Wl,-framework','-Wl,vecLib'])
             if args:
                 self.set_info(extra_compile_args=args,
@@ -1269,7 +1371,7 @@ class blas_opt_info(system_info):
         if atlas_info:
             version_info = atlas_info.copy()
             atlas_version = get_atlas_version(**version_info)
-            if not atlas_info.has_key('define_macros'):
+            if 'define_macros' not in atlas_info:
                 atlas_info['define_macros'] = []
             if atlas_version is None:
                 atlas_info['define_macros'].append(('NO_ATLAS_INFO',2))
@@ -1347,6 +1449,7 @@ class blas_src_info(system_info):
         srotmg zdrot cdotu daxpy drotm idamax scopy sscal zdscal crotg
         dcabs1 drotmg isamax sdot sswap zrotg cscal dcopy dscal izamax
         snrm2 zaxpy zscal csrot ddot dswap sasum srot zcopy zswap
+        scabs1
         '''
         blas2 = '''
         cgbmv chpmv ctrsv dsymv dtrsv sspr2 strmv zhemv ztpmv cgemv
@@ -1365,6 +1468,7 @@ class blas_src_info(system_info):
         sources = [os.path.join(src_dir,f+'.f') \
                    for f in (blas1+blas2+blas3).split()]
         #XXX: should we check here actual existence of source files?
+        sources = [f for f in sources if os.path.isfile(f)]
         info = {'sources':sources,'language':'f77'}
         self.set_info(**info)
 
@@ -1404,7 +1508,6 @@ class _numpy_info(system_info):
     notfounderror = NumericNotFoundError
 
     def __init__(self):
-        from distutils.sysconfig import get_python_inc
         include_dirs = []
         try:
             module = __import__(self.modulename)
@@ -1413,10 +1516,11 @@ class _numpy_info(system_info):
                 if name=='lib':
                     break
                 prefix.append(name)
-            include_dirs.append(get_python_inc(prefix=os.sep.join(prefix)))
+            include_dirs.append(distutils.sysconfig.get_python_inc(
+                                        prefix=os.sep.join(prefix)))
         except ImportError:
             pass
-        py_incl_dir = get_python_inc()
+        py_incl_dir = distutils.sysconfig.get_python_inc()
         include_dirs.append(py_incl_dir)
         for d in default_include_dirs:
             d = os.path.join(d, os.path.basename(py_incl_dir))
@@ -1478,7 +1582,6 @@ class numpy_info(_numpy_info):
 class numerix_info(system_info):
     section = 'numerix'
     def calc_info(self):
-        import sys, os
         which = None, None
         if os.getenv("NUMERIX"):
             which = os.getenv("NUMERIX"), "environment var"
@@ -1497,14 +1600,15 @@ class numerix_info(system_info):
                         import numarray
                         which = "numarray", "defaulted"
                     except ImportError,msg3:
-                        print msg1
-                        print msg2
-                        print msg3
+                        log.info(msg1)
+                        log.info(msg2)
+                        log.info(msg3)
         which = which[0].strip().lower(), which[1]
         if which[0] not in ["numeric", "numarray", "numpy"]:
             raise ValueError("numerix selector must be either 'Numeric' "
                              "or 'numarray' or 'numpy' but the value obtained"
                              " from the %s was '%s'." % (which[1], which[0]))
+        os.environ['NUMERIX'] = which[0]
         self.set_info(**get_info(which[0]))
 
 class f2py_info(system_info):
@@ -1530,7 +1634,6 @@ class boost_python_info(system_info):
         return [ d for d in dirs if os.path.isdir(d) ]
 
     def calc_info(self):
-        from distutils.sysconfig import get_python_inc
         src_dirs = self.get_src_dirs()
         src_dir = ''
         for d in src_dirs:
@@ -1539,7 +1642,7 @@ class boost_python_info(system_info):
                 break
         if not src_dir:
             return
-        py_incl_dir = get_python_inc()
+        py_incl_dir = distutils.sysconfig.get_python_inc()
         srcs_dir = os.path.join(src_dir,'libs','python','src')
         bpl_srcs = glob(os.path.join(srcs_dir,'*.cpp'))
         bpl_srcs += glob(os.path.join(srcs_dir,'*','*.cpp'))
@@ -1597,7 +1700,7 @@ class _pkg_config_info(system_info):
     cflags_flag = '--cflags'
 
     def get_config_exe(self):
-        if os.environ.has_key(self.config_env_var):
+        if self.config_env_var in os.environ:
             return os.environ[self.config_env_var]
         return self.default_config_exe
     def get_config_output(self, config_exe, option):
@@ -1607,9 +1710,9 @@ class _pkg_config_info(system_info):
 
     def calc_info(self):
         config_exe = find_executable(self.get_config_exe())
-        if not os.path.isfile(config_exe):
-            print 'File not found: %s. Cannot determine %s info.' \
-                  % (config_exe, self.section)
+        if not config_exe:
+            log.warn('File not found: %s. Cannot determine %s info.' \
+                  % (config_exe, self.section))
             return
         info = {}
         macros = []
@@ -1819,8 +1922,7 @@ def combine_paths(*args,**kws):
     else:
         result = combine_paths(*(combine_paths(args[0],args[1])+args[2:]))
     verbosity = kws.get('verbosity',1)
-    if verbosity>1 and result:
-        print '(','paths:',','.join(result),')'
+    log.debug('(paths: %s)', ','.join(result))
     return result
 
 language_map = {'c':0,'c++':1,'f77':2,'f90':3}
@@ -1831,7 +1933,7 @@ def dict_append(d,**kws):
         if k=='language':
             languages.append(v)
             continue
-        if d.has_key(k):
+        if k in d:
             if k in ['library_dirs','include_dirs','define_macros']:
                 [d[k].append(vv) for vv in v if vv not in d[k]]
             else:
@@ -1843,25 +1945,46 @@ def dict_append(d,**kws):
         d['language'] = l
     return
 
-def show_all():
-    import system_info
-    import pprint
-    match_info = re.compile(r'.*?_info').match
+def parseCmdLine(argv=(None,)):
+    import optparse
+    parser = optparse.OptionParser("usage: %prog [-v] [info objs]")
+    parser.add_option('-v', '--verbose', action='store_true', dest='verbose',
+                      default=False,
+                      help='be verbose and print more messages')
+
+    opts, args = parser.parse_args(args=argv[1:])
+    return opts, args
+
+def show_all(argv=None):
+    import inspect
+    if argv is None:
+        argv = sys.argv
+    opts, args = parseCmdLine(argv)
+    if opts.verbose:
+        log.set_threshold(log.DEBUG)
+    else:
+        log.set_threshold(log.INFO)
     show_only = []
-    for n in sys.argv[1:]:
+    for n in args:
         if n[-5:] != '_info':
             n = n + '_info'
         show_only.append(n)
     show_all = not show_only
-    for n in filter(match_info,dir(system_info)):
-        if n in ['system_info','get_info']: continue
+    _gdict_ = globals().copy()
+    for name, c in _gdict_.iteritems():
+        if not inspect.isclass(c):
+            continue
+        if not issubclass(c, system_info) or c is system_info:
+            continue
         if not show_all:
-            if n not in show_only: continue
-            del show_only[show_only.index(n)]
-        c = getattr(system_info,n)()
-        c.verbosity = 2
-        r = c.get_info()
+            if name not in show_only:
+                continue
+            del show_only[show_only.index(name)]
+        conf = c()
+        conf.verbosity = 2
+        r = conf.get_info()
     if show_only:
-        print 'Info classes not defined:',','.join(show_only)
+        log.info('Info classes not defined: %s',','.join(show_only))
+
 if __name__ == "__main__":
     show_all()
