@@ -15,36 +15,13 @@ from numpy.lib.getlimits import finfo
 from numpy.lib.twodim_base import diag, vander
 from numpy.lib.shape_base import hstack, atleast_1d
 from numpy.lib.function_base import trim_zeros, sort_complex
-eigvals = None
-lstsq = None
+from numpy.linalg import eigvals, lstsq
 
 class RankWarning(UserWarning):
     """Issued by polyfit when Vandermonde matrix is rank deficient.
     """
     pass
 
-def get_linalg_funcs():
-    "Look for linear algebra functions in numpy"
-    global eigvals, lstsq
-    from numpy.dual import eigvals, lstsq
-    return
-
-def _eigvals(arg):
-    "Return the eigenvalues of the argument"
-    try:
-        return eigvals(arg)
-    except TypeError:
-        get_linalg_funcs()
-        return eigvals(arg)
-
-def _lstsq(X, y, rcond):
-    "Do least squares on the arguments"
-    try:
-        return lstsq(X, y, rcond)
-    except TypeError:
-        get_linalg_funcs()
-        return lstsq(X, y, rcond)
-
 def poly(seq_of_zeros):
     """
     Return polynomial coefficients given a sequence of roots.
@@ -94,7 +71,7 @@ def poly(seq_of_zeros):
     seq_of_zeros = atleast_1d(seq_of_zeros)
     sh = seq_of_zeros.shape
     if len(sh) == 2 and sh[0] == sh[1]:
-        seq_of_zeros = _eigvals(seq_of_zeros)
+        seq_of_zeros = eigvals(seq_of_zeros)
     elif len(sh) ==1:
         pass
     else:
@@ -177,7 +154,7 @@ def roots(p):
         # build companion matrix and find its eigenvalues (the roots)
         A = diag(NX.ones((N-2,), p.dtype), -1)
         A[0, :] = -p[1:] / p[0]
-        roots = _eigvals(A)
+        roots = eigvals(A)
     else:
         roots = NX.array([])
 
@@ -262,17 +239,20 @@ def polyint(p, m=1, k=None):
     if len(k) < m:
         raise ValueError, \
               "k must be a scalar or a rank-1 array of length 1 or >m."
+
+    truepoly = isinstance(p, poly1d)
+    p = NX.asarray(p) + 0.0
     if m == 0:
+        if truepoly:
+            return poly1d(p)
         return p
     else:
-        truepoly = isinstance(p, poly1d)
-        p = NX.asarray(p)
-        y = NX.zeros(len(p)+1, float)
+        y = NX.zeros(len(p) + 1, p.dtype)
         y[:-1] = p*1.0/NX.arange(len(p), 0, -1)
         y[-1] = k[0]
-        val = polyint(y, m-1, k=k[1:])
+        val = polyint(y, m - 1, k=k[1:])
         if truepoly:
-            val = poly1d(val)
+            return poly1d(val)
         return val
 
 def polyder(p, m=1):
@@ -500,7 +480,7 @@ def polyfit(x, y, deg, rcond=None, full=False):
 
     # solve least squares equation for powers of x
     v = vander(x, order)
-    c, resids, rank, s = _lstsq(v, y, rcond)
+    c, resids, rank, s = lstsq(v, y, rcond)
 
     # warn on rank reduction, which indicates an ill conditioned matrix
     if rank != order and not full:
@@ -641,7 +621,7 @@ def polysub(a1, a2):
 
     Examples
     --------
-    .. math:: (2 x^2 + 10x - 2) - (3 x^2 + 10x -4) = (-x^2 + 2)
+    .. math:: (2 x^2 + 10 x - 2) - (3 x^2 + 10 x -4) = (-x^2 + 2)
 
     >>> np.polysub([2, 10, -2], [3, 10, -4])
     array([-1,  0,  2])
@@ -733,12 +713,14 @@ def polydiv(u, v):
 
     """
     truepoly = (isinstance(u, poly1d) or isinstance(u, poly1d))
-    u = atleast_1d(u)
-    v = atleast_1d(v)
+    u = atleast_1d(u) + 0.0
+    v = atleast_1d(v) + 0.0
+    # w has the common type
+    w = u[0] + v[0]
     m = len(u) - 1
     n = len(v) - 1
     scale = 1. / v[0]
-    q = NX.zeros((max(m-n+1,1),), float)
+    q = NX.zeros((max(m - n + 1, 1),), w.dtype)
     r = u.copy()
     for k in range(0, m-n+1):
         d = scale * r[k]
@@ -747,8 +729,7 @@ def polydiv(u, v):
     while NX.allclose(r[0], 0, rtol=1e-14) and (r.shape[-1] > 1):
         r = r[1:]
     if truepoly:
-        q = poly1d(q)
-        r = poly1d(r)
+        return poly1d(q), poly1d(r)
     return q, r
 
 _poly_mat = re.compile(r"[*][*]([0-9]*)")
