@@ -1,11 +1,12 @@
 """ Test functions for linalg module
 """
 
+import numpy as np
 from numpy.testing import *
 from numpy import array, single, double, csingle, cdouble, dot, identity
 from numpy import multiply, atleast_2d, inf, asarray, matrix
 from numpy import linalg
-from numpy.linalg import matrix_power
+from numpy.linalg import matrix_power, norm
 
 def ifthen(a, b):
     return not a or b
@@ -202,7 +203,7 @@ class TestBoolPower(TestCase):
         assert_equal(matrix_power(A,2),A)
 
 
-class HermitianTestCase:
+class HermitianTestCase(object):
     def test_single(self):
         a = array([[1.,2.], [2.,1.]], dtype=single)
         self.do(a)
@@ -239,23 +240,78 @@ class HermitianTestCase:
 
 class TestEigvalsh(HermitianTestCase, TestCase):
     def do(self, a):
+        # note that eigenvalue arrays must be sorted since
+        # their order isn't guaranteed.
         ev = linalg.eigvalsh(a)
-        # flip resulting eigenvalue array, since they are returned in
-        # reverse order from the values given by linal.eig
-        ev = ev[::-1]
-
         evalues, evectors = linalg.eig(a)
+        ev.sort()
+        evalues.sort()
         assert_almost_equal(ev, evalues)
 
 class TestEigh(HermitianTestCase, TestCase):
     def do(self, a):
+        # note that eigenvalue arrays must be sorted since
+        # their order isn't guaranteed.
         ev, evc = linalg.eigh(a)
-        # flip resulting eigenvalue array, since they are returned in
-        # reverse order from the values given by linal.eig
-        ev = ev[::-1]
-
         evalues, evectors = linalg.eig(a)
+        ev.sort()
+        evalues.sort()
         assert_almost_equal(ev, evalues)
 
+class _TestNorm(TestCase):
+    dt = None
+    dec = None
+    def test_empty(self):
+        assert_equal(norm([]), 0.0)
+        assert_equal(norm(array([], dtype=self.dt)), 0.0)
+        assert_equal(norm(atleast_2d(array([], dtype=self.dt))), 0.0)
+
+    def test_vector(self):
+        a = [1.0,2.0,3.0,4.0]
+        b = [-1.0,-2.0,-3.0,-4.0]
+        c = [-1.0, 2.0,-3.0, 4.0]
+
+        def _test(v):
+            np.testing.assert_almost_equal(norm(v), 30**0.5, decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,inf), 4.0, decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,-inf), 1.0, decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,1), 10.0, decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,-1), 12.0/25,
+                    decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,2), 30**0.5,
+                    decimal=self.dec)
+            np.testing.assert_almost_equal(norm(v,-2), ((205./144)**-0.5),
+                    decimal=self.dec)
+
+        for v in (a, b, c,):
+            _test(v)
+
+        for v in (array(a, dtype=self.dt), array(b, dtype=self.dt),
+                  array(c, dtype=self.dt)):
+            _test(v)
+
+    def test_matrix(self):
+        A = matrix([[1.,3.],[5.,7.]], dtype=self.dt)
+        A = matrix([[1.,3.],[5.,7.]], dtype=self.dt)
+        assert_almost_equal(norm(A), 84**0.5)
+        assert_almost_equal(norm(A,'fro'), 84**0.5)
+        assert_almost_equal(norm(A,inf), 12.0)
+        assert_almost_equal(norm(A,-inf), 4.0)
+        assert_almost_equal(norm(A,1), 10.0)
+        assert_almost_equal(norm(A,-1), 6.0)
+        assert_almost_equal(norm(A,2), 9.1231056256176615)
+        assert_almost_equal(norm(A,-2), 0.87689437438234041)
+
+        self.assertRaises(ValueError, norm, A, 'nofro')
+        self.assertRaises(ValueError, norm, A, -3)
+
+class TestNormDouble(_TestNorm):
+    dt = np.double
+    dec= 12
+
+class TestNormSingle(_TestNorm):
+    dt = np.float32
+    dec = 6
+
 if __name__ == "__main__":
     run_module_suite()
