@@ -381,6 +381,16 @@ static int PyArray_DescrAlignConverter2 \
        (PyObject *, PyArray_Descr **);
 static int PyArray_SearchsideConverter \
        (PyObject *, void *);
+static PyObject * PyArray_CheckAxis \
+       (PyArrayObject *, int *, int);
+static npy_intp PyArray_OverflowMultiplyList \
+       (register npy_intp *, register int);
+static int PyArray_CompareString \
+       (char *, char *, size_t);
+static PyObject * PyArray_MultiIterFromObjects \
+       (PyObject **, int, int, ...);
+static int PyArray_GetEndianness \
+       (void);
 
 #else
 
@@ -937,11 +947,27 @@ static void **PyArray_API=NULL;
 #define PyArray_SearchsideConverter \
         (*(int (*)(PyObject *, void *)) \
          PyArray_API[205])
+#define PyArray_CheckAxis \
+        (*(PyObject * (*)(PyArrayObject *, int *, int)) \
+         PyArray_API[206])
+#define PyArray_OverflowMultiplyList \
+        (*(npy_intp (*)(register npy_intp *, register int)) \
+         PyArray_API[207])
+#define PyArray_CompareString \
+        (*(int (*)(char *, char *, size_t)) \
+         PyArray_API[208])
+#define PyArray_MultiIterFromObjects \
+        (*(PyObject * (*)(PyObject **, int, int, ...)) \
+         PyArray_API[209])
+#define PyArray_GetEndianness \
+        (*(int (*)(void)) \
+         PyArray_API[210])
 
 #if !defined(NO_IMPORT_ARRAY) && !defined(NO_IMPORT)
 static int
 _import_array(void)
 {
+  int st;
   PyObject *numpy = PyImport_ImportModule("numpy.core.multiarray");
   PyObject *c_api = NULL;
   if (numpy == NULL) return -1;
@@ -960,6 +986,30 @@ _import_array(void)
         (int) NPY_VERSION, (int) PyArray_GetNDArrayCVersion());
     return -1;
   }
+ 
+  /* 
+   * Perform runtime check of endianness and check it matches the one set by
+   * the headers (npy_endian.h) as a safeguard 
+   */
+  st = PyArray_GetEndianness();
+  if (st == NPY_CPU_UNKNOWN_ENDIAN) {
+    PyErr_Format(PyExc_RuntimeError, "FATAL: module compiled as unknown endian");
+    return -1;
+  }
+#ifdef NPY_BIG_ENDIAN
+  if (st != NPY_CPU_BIG) {
+    PyErr_Format(PyExc_RuntimeError, "FATAL: module compiled as "\
+        "big endian, but detected different endianness at runtime");
+    return -1;
+  }
+#elif defined(NPY_LITTLE_ENDIAN)
+  if (st != NPY_CPU_LITTLE) {
+    PyErr_Format(PyExc_RuntimeError, "FATAL: module compiled as"\
+        "little endian, but detected different endianness at runtime");
+    return -1;
+  }
+#endif
+
   return 0;
 }
 
