@@ -19,6 +19,20 @@ typedef struct {
         void *ptr;
         PyObject *obj;
 	PyObject *userloops;
+    
+        /* generalized ufunc */
+        int core_enabled;      /* 0 for scalar ufunc; 1 for generalized ufunc */
+        int core_num_dim_ix;   /* number of distinct dimension names in
+                                  signature */
+ 
+        /* dimension indices of input/output argument k are stored in
+           core_dim_ixs[core_offsets[k]..core_offsets[k]+core_num_dims[k]-1] */
+        int *core_num_dims;    /* numbers of core dimensions of each argument */
+        int *core_dim_ixs;     /* dimension indices in a flatted form; indices
+                                  are in the range of [0,core_num_dim_ix) */
+        int *core_offsets;     /* positions of 1st core dimensions of each
+                                  argument in core_dim_ixs */
+        char *core_signature;  /* signature string for printing purpose */
 } PyUFuncObject;
 
 #include "arrayobject.h"
@@ -122,6 +136,11 @@ typedef struct {
         int notimplemented; /* The loop caused notimplemented */
         int objfunc; /* This loop calls object functions
                         (an inner-loop function with argument types */
+    
+        /* generalized ufunc */
+        npy_intp *core_dim_sizes;   /* stores sizes of core dimensions;
+                                       contains 1 + core_num_dim_ix elements */
+        npy_intp *core_strides;     /* strides of loop and core dimensions */
 } PyUFuncLoopObject;
 
 /* Could make this more clever someday */
@@ -263,11 +282,6 @@ typedef struct _loop1d_info {
 		| ((SW_INVALID & fpstatus) ? UFUNC_FPE_INVALID : 0);	\
 	}
 
-#define isnan(x) (_isnan((double)(x)))
-#define isinf(x) ((_fpclass((double)(x)) == _FPCLASS_PINF) ||	\
-		  (_fpclass((double)(x)) == _FPCLASS_NINF))
-#define isfinite(x) (_finite((double) x))
-
 /* Solaris --------------------------------------------------------*/
 /* --------ignoring SunOS ieee_flags approach, someone else can
 **         deal with that! */
@@ -285,6 +299,23 @@ typedef struct _loop1d_info {
 	(void) fpsetsticky(0);						\
 	}
 
+#elif defined(__MINGW32__) && defined(__amd64__)
+#include "mingw_amd64_fenv.h"
+
+#define UFUNC_CHECK_STATUS(ret) {                                       \
+	int fpstatus = (int) npy_fetestexcept(NPY_FE_DIVBYZERO | 	\
+		NPY_FE_OVERFLOW | NPY_FE_UNDERFLOW | NPY_FE_INVALID);	\
+	ret = ((NPY_FE_DIVBYZERO  & fpstatus) ? UFUNC_FPE_DIVIDEBYZERO : 0) \
+		| ((NPY_FE_OVERFLOW   & fpstatus) ? UFUNC_FPE_OVERFLOW : 0)	\
+		| ((NPY_FE_UNDERFLOW  & fpstatus) ? UFUNC_FPE_UNDERFLOW : 0) \
+		| ((NPY_FE_INVALID    & fpstatus) ? UFUNC_FPE_INVALID : 0);	\
+	(void) npy_feclearexcept(NPY_FE_DIVBYZERO | NPY_FE_OVERFLOW |		\
+			     NPY_FE_UNDERFLOW | NPY_FE_INVALID);		\
+}
+
+#define generate_divbyzero_error() npy_feraiseexcept(NPY_FE_DIVBYZERO)
+#define generate_overflow_error() npy_feraiseexcept(NPY_FE_OVERFLOW)
+
 #elif defined(__GLIBC__) || defined(__APPLE__) || defined(__CYGWIN__) || defined(__MINGW32__) || (defined(__FreeBSD__) && (__FreeBSD_version >= 502114))
 
 #if defined(__GLIBC__) || defined(__APPLE__) || defined(__MINGW32__) || defined(__FreeBSD__)
