@@ -35,7 +35,6 @@ last revision: 07.01.2007
 __all__ = ['ediff1d', 'unique1d', 'intersect1d', 'intersect1d_nu', 'setxor1d',
            'setmember1d', 'union1d', 'setdiff1d']
 
-import time
 import numpy as np
 
 def ediff1d(ary, to_end=None, to_begin=None):
@@ -58,8 +57,13 @@ def ediff1d(ary, to_end=None, to_begin=None):
     ed : array
         The differences. Loosely, this will be (ary[1:] - ary[:-1]).
 
+    Notes
+    -----
+    When applied to masked arrays, this function drops the mask information
+    if the `to_begin` and/or `to_end` parameters are used
+
     """
-    ary = np.asarray(ary).flat
+    ary = np.asanyarray(ary).flat
     ed = ary[1:] - ary[:-1]
     arrays = [ed]
     if to_begin is not None:
@@ -133,7 +137,7 @@ def unique1d(ar1, return_index=False, return_inverse=False):
                       "the output was (indices, unique_arr), but "
                       "has now been reversed to be more consistent.")
 
-    ar = np.asarray(ar1).flatten()
+    ar = np.asanyarray(ar1).flatten()
     if ar.size == 0:
         if return_inverse and return_index:
             return ar, np.empty(0, np.bool), np.empty(0, np.bool)
@@ -280,11 +284,22 @@ def setmember1d(ar1, ar2):
     mask : ndarray, bool
         The values `ar1[mask]` are in `ar2`.
 
+
     See Also
     --------
     numpy.lib.arraysetops : Module with a number of other functions for
                             performing set operations on arrays.
 
+    Examples
+    --------
+    >>> test = np.arange(5)
+    >>> states = [0, 2]
+    >>> mask = np.setmember1d(test,states)
+    >>> mask
+    array([ True, False,  True, False, False], dtype=bool)
+    >>> test[mask]
+    array([0, 2])
+
     """
     ar1 = np.asarray( ar1 )
     ar2 = np.asarray( ar2 )
